local Tasks = Tasks
local L = Tasks.L
local TasksUtils = TasksUtils
local db
local editListTasksArgs

Tasks:RegisterCallback("Tasks-Initialize", "ConfigInit")

function Tasks:ConfigInit()
  db = Tasks.db
  self:RegisterChatCommand("tasks", "ChatCommandReceived")
  
  --sets up all the config dialogs and popups
  LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable(
    "Tasks", 
    {
      type = "group",
      name = "Options",
      inline=true,
      args = {
        addonEnabledGroup = {
          type = "group",
          name = "Addon Enabled",
          inline=true,
          order = 10,
          args = {
            addonEnabled = {
              name = "Enable Tasks",
              desc = "Enables / disables Tasks",
              type = "toggle",
              set = function(info,val) db.global.addonEnabled = val end,
              get = function(info) return db.global.addonEnabled  end,
            },
          },
        },
        displayOptionsGroup = {
          type = "group",
          name = "Display Options",
          inline=true,
          order = 20,
          args = {
            lockDisplay = {
              name = "Lock Display",
              desc = "Enable / disable moving the Tasks list on screen",
              type = "toggle",
              set = function(info,val) 
                Tasks.TasksFrame:EnableMouse(not val);
                db.global.displayOptions.frameSettings.locked = val; 
              end,
              get = function(info) return db.global.displayOptions.frameSettings.locked  end,
            },
          },
        },
        resetOptionsGroup = {
          type = "group",
          name = "Reset Options",
          inline=true,
          order = 30,
          args = {
            weeklyResetDay = {
              name = L["weeklyResetDay"],
              desc = L["weeklyResetDesc"],
              type = "select",
              values = {
                [Tasks.constants.SUNDAY] = L["sunday"], 
                [Tasks.constants.MONDAY] = L["monday"], 
                [Tasks.constants.TUESDAY] = L["tuesday"], 
                [Tasks.constants.WEDNESDAY] = L["wednesday"], 
                [Tasks.constants.THURSDAY] = L["thursday"], 
                [Tasks.constants.FRIDAY] = L["friday"], 
                [Tasks.constants.SATURDAY] = L["saturday"]
              },
              order = 10,
              disabled = function(info) return not db.global.addonEnabled end,
              set = function(info,val) db.global.resetOptions.weeklyResetDay = tonumber(val) end,
              get = function(info) return db.global.resetOptions.weeklyResetDay end,
            },
            --dailyResetHour = {
            --  name = L["dailyResetHour"],
            --  desc = L["dailyResetDesc"],
            --  type = "select",
            --  values = {
            --    [Tasks.constants.SUNDAY] = L["sunday"], 
            --    [Tasks.constants.MONDAY] = L["monday"], 
            --    [Tasks.constants.TUESDAY] = L["tuesday"], 
            --    [Tasks.constants.WEDNESDAY] = L["wednesday"], 
            --    [Tasks.constants.THURSDAY] = L["thursday"], 
            --    [Tasks.constants.FRIDAY] = L["friday"], 
            --    [Tasks.constants.SATURDAY] = L["saturday"]
            --  },
            --  order = 10,
            --  disabled = function(info) return not db.global.addonEnabled end,
            --  set = function(info,val) db.global.resetOptions.weeklyResetDay = tonumber(val) end,
            --  get = function(info) return db.global.resetOptions.weeklyResetDay end,
            --},
          },
        },
      },
    }
  )
  LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Tasks", L["tasks"])
  
  local taskListsOptionsTable = {
    type = "group",
    name = L["taskLists"],
    childGroups = "tab",
    disabled = function(info) return not db.global.addonEnabled end,
	handler = Tasks,
    args={
      listSelection = {
        type = "select",
        name = L["selectList"],
        order = 10,
        values = function()
                   local listNames = {} 
                   for k,v in pairs(db.global.lists) do
                     listNames[k] = v.name
                   end
                   return listNames
                 end,
        set = function(info,val) db.global.editList = val Tasks:updateEditListTasksArgs() end,
        get = function(info) return db.global.editList end,
      },
      newList = {
        type = "execute",
        name = L["createNewList"] ,
        order = 20,
        func = "popupNewList",
      },
      noListsGroup = {
        type = "group",
        name = "",
        order = 40,
        inline=true,
        hidden = function(info) return db.global.editList ~= nil end,
        args = {
          noLists = {
            type = "description",
            name = L["noListSelected"],
            fontSize = "medium",
            order = 10,
            hidden = function(info) return db.global.editList ~= nil end,
          },
        },
      },
      editListOptionsGroup = {
        type = "group",
        name = L["listOptions"],
        order = 40,
        hidden = function(info) return db.global.editList == nil end,
        args = {
          editListName = {
            type = "input",
            name = L["name"],
            order = 10,
            set = function(info,val) db.global.lists[db.global.editList].name = val end,
            get = function(info) return db.global.lists[db.global.editList].name end,
          },
          editListResetFrequency = {
            type = "select",
            name = L["resetFrequency"],
            order = 20,
            values = {
              [Tasks.constants.DAILY_RESET] = L["daily"], 
              [Tasks.constants.WEEKLY_RESET] = L["weekly"], 
              [Tasks.constants.NEVER_RESET] = L["never"]
            },
            style = "radio",
            set = function(info, val) db.global.lists[db.global.editList].resetFrequency = val end,
            get = function(info) return db.global.lists[db.global.editList].resetFrequency end,
          },
          deleteList = {
            type = "execute",
            name = L["deleteList"],
            order = 100,
            func = "deleteEditList",
            confirm = function(info) return L["confirmListDelete"]:format(db.global.lists[db.global.editList].name) end
          },
        },
      },
      editListTasksGroup = {
        type = "group",
        name = L["tasks"],
        order = 40,
        hidden = function(info) return db.global.editList == nil end,
		childGroups = "tree",
        args = { },
      },
    },
  }
  editListTasksArgs = taskListsOptionsTable.args.editListTasksGroup.args
  Tasks:updateEditListTasksArgs()
  LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("Tasks-TaskLists", taskListsOptionsTable)
  LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Tasks-TaskLists", L["taskLists"], L["tasks"])
  
  LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable(
    "Tasks-TaskItems", 
    {type = "group",
      name = "Tasks",
      args={
      },
    }
  )
  LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Tasks-TaskItems", L["tasks"], L["tasks"])
 
  if not StaticPopupDialogs["TASKS_CREATENEWTASKLIST"] then
    StaticPopupDialogs["TASKS_CREATENEWTASKLIST"] = {
      text = L["promptNewTaskList"],
      button1 = ACCEPT,
      button2 = CANCEL,
      OnAccept = function(self)
        Tasks:addNewList(self.editBox:GetText())
      end,
      OnShow = function(self)
        self.button1:Disable()
      end,
      EditBoxOnTextChanged = function (self, data)
        if self:GetText():len() > 1 then
          self:GetParent().button1:Enable()
        else
          self:GetParent().button1:Disable()
        end
      end,
      EditBoxOnEnterPressed = function(self)
        if ( self:GetParent().button1:IsEnabled() ) then
          Tasks:addNewList(self:GetParent().editBox:GetText())
          self:GetParent():Hide()
        end
      end,
      EditBoxOnEscapePressed = function(self)
        self:GetParent():Hide()
      end,
      timeout = 0,
      whileDead = true,
      hideOnEscape = true,
      preferredIndex = 3,
      hasEditBox = true,
      hideOnEscape = true,
    }
  end
  
  if not StaticPopupDialogs["TASKS_CREATENEWTASK"] then
    StaticPopupDialogs["TASKS_CREATENEWTASK"] = {
      text = L["promptNewTask"],
      button1 = ACCEPT,
      button2 = CANCEL,
      OnAccept = function(self)
        Tasks:addNewTask(self.editBox:GetText(), db.global.editList)
      end,
      OnShow = function(self)
        self.button1:Disable()
      end,
      EditBoxOnTextChanged = function (self, data)
        if self:GetText():len() > 1 then
          self:GetParent().button1:Enable()
        else
          self:GetParent().button1:Disable()
        end
      end,
      EditBoxOnEnterPressed = function(self)
        if ( self:GetParent().button1:IsEnabled() ) then
          Tasks:addNewTask(self:GetParent().editBox:GetText(), db.global.editList)
          self:GetParent():Hide()
        end
      end,
      EditBoxOnEscapePressed = function(self)
        self:GetParent():Hide()
      end,
      timeout = 0,
      whileDead = true,
      hideOnEscape = true,
      preferredIndex = 3,
      hasEditBox = true,
      hideOnEscape = true,
    }
  end
  
  --first time open to category fails 
  --just opens interface options, doesn't route), 
  --so open and close interface options on 
  --load to make it work when /tasks is typed....
  InterfaceOptionsFrame_Show()
  InterfaceOptionsFrame_Show()
end

function Tasks:updateEditListTasksArgs()
  if (db.global.editList) then
    local args = editListTasksArgs
    local taskList = db.global.lists[db.global.editList].tasks
    
    for k in pairs(args) do
      args[k] = nil
    end
    
    args.newTask = {
      type = "execute",
      name = L["createNewTask"],
      order = 10,
      func = "popupNewTask",
    }
    args.reorderTasks = {
      type = "execute",
      name = L["reorderTasks"],
      order = 20,
      func = "reorderTasks",
    }
    
    if(db.global.lists[db.global.editList].tasks) then 
      for k,v in pairs(db.global.lists[db.global.editList].tasks) do 
        args[k.."group"] = {}
        local t = args[k.."group"]
        t.type = "group"
        t.name = function(info) return taskList[k].name end
        t.order = v.order +20
	    t.inline = false
        t.args = {
          editTaskName = {
            type = "input",
            name = L["name"],
            order = 10,
            set = function(info,val) taskList[k].name = val end,
            get = function(info) return taskList[k].name end,
          },
          deleteTask = {
            type = "execute",
            name = L["deleteTask"],
            order = 100,
            func = "deleteTask",
            confirm = function(info) return L["confirmTaskDelete"]:format(taskList[k].name) end,
            arg = k
          },
        }
      end
    end
  end
end

function Tasks:ChatCommandReceived(msg)
  InterfaceOptionsFrame_OpenToCategory("Task Lists")
end

function Tasks:popupNewList()
  StaticPopup_Show("TASKS_CREATENEWTASKLIST")
end

function Tasks:popupNewTask()
  StaticPopup_Show("TASKS_CREATENEWTASK")
end

function Tasks:deleteEditList()
  db.global.lists[db.global.editList] = nil
  db.global.editList = nil
end

function Tasks:deleteTask(info)
  db.global.lists[db.global.editList].tasks[info.arg] = nil
  db.global.lists[db.global.editList].taskCount = db.global.lists[db.global.editList].taskCount - 1
  self:updateEditListTasksArgs()
end

function Tasks:addNewList(newName)
  local newId = "list"..self:NextId()
  db.global.lists[newId].name = newName;
  db.global.lists[newId].id = newId;
  db.global.editList = newId
  self:updateEditListTasksArgs()
  LibStub("AceConfigRegistry-3.0"):NotifyChange("Tasks-TaskLists")
end

function Tasks:addNewTask(newName)
  local newId = "task"..self:NextId()
  local list = db.global.lists[db.global.editList]
  list.tasks[newId].name = newName;
  list.tasks[newId].id = newId;
  list.tasks[newId].listId = db.global.editList;
  list.tasks[newId].order = list.taskCount
  list.taskCount = list.taskCount + 1
  self:updateEditListTasksArgs()
  LibStub("AceConfigRegistry-3.0"):NotifyChange("Tasks-TaskLists")
end